package server

import (
	. "contract_backend/constant"
	"contract_backend/util"
	"fmt"
	"io"
	"time"

	"github.com/btcsuite/btcd/btcec"
	"github.com/ethereum/go-ethereum/common"
	"github.com/ethereum/go-ethereum/crypto"
	"github.com/gin-gonic/gin"
	log "github.com/sirupsen/logrus"
	"github.com/tidwall/gjson"
)

func userChange(c *gin.Context) {
	// 获取原始json数据
	rawJson, err := io.ReadAll(c.Request.Body)
	if err != nil {
		log.WithError(err).Error("read body error")
		c.JSON(200, withError(InvalidParam))
		return
	}

	// 判断是插入还是update
	table := gjson.Get(string(rawJson), "table").String()
	if table != "users" {
		log.WithField("table", table).Error("wrong table")
		c.JSON(200, withError(InvalidParam))
		return
	}

	var userId string
	var key *btcec.PrivateKey

	operation := gjson.Get(string(rawJson), "operation").String()
	switch operation {
	case "UPDATE":
		userId = gjson.Get(string(rawJson), "new_row.id").String()
		key, err = util.GetPrivateKeyByUserId(conf.Chain.XPrivateKey, userId)
		if err != nil {
			log.WithError(err).Error("get private key error")
			c.JSON(200, withError(InternalError))
			return
		}
	default:
		log.WithField("operation", operation).Error("wrong operation")
		c.JSON(200, withError(InvalidParam))
		return
	}

	needLogin := true

	if operation == "UPDATE" {
		lastLogin := gjson.Get(string(rawJson), "old_row.last_sign_in_at").String()
		newLogin := gjson.Get(string(rawJson), "new_row.last_sign_in_at").String()
		if lastLogin == newLogin {
			needLogin = false
		} else if lastLogin != "" {
			fmt.Println(lastLogin)
			loginTime, err := time.Parse(lastLogin, "2006-01-02T15:04:05.999999+07:00")
			if err != nil {
				log.WithError(err).Error("parse time error")
				c.JSON(200, withError(InternalError))
				return
			}

			// 如果loginTime是0点之前
			todayUnix := time.Now().Unix() % 86400
			lastLoginUnix := loginTime.Unix() % 86400
			if lastLoginUnix >= todayUnix {
				needLogin = false
			}
		}
	}

	if !needLogin {
		c.JSON(200, withSuccess(""))
		return
	}

	// 需要登录
	userAddress := crypto.PubkeyToAddress(key.PublicKey)
	val, err := d.EstimateGasRecharge(userAddress)
	if err != nil {
		log.WithError(err).Error("estimate gas error")
		c.JSON(200, withError(InternalError))
		return
	}

	if val != nil {
		// 需要充值
		txHash, err := d.TransferETH(conf.Chain.GasSenderPrivateKey, userAddress, val)
		if err != nil {
			log.WithError(err).Error("transfer eth error")
			c.JSON(200, withError(InternalError))
			return
		}

		receipt, err := d.WaitForReceipt(txHash)
		if err != nil {
			log.WithError(err).Error("wait for receipt error")
			c.JSON(200, withError(InternalError))
			return
		}
		if receipt.Status != 1 {
			log.WithField("receipt", receipt).Error("receipt status error")
			c.JSON(200, withError(InternalError))
			return
		}

		log.WithFields(log.Fields{
			"userId":  userId,
			"address": userAddress.String(),
			"txHash":  txHash,
			"val":     val.String(),
		}).Info("user recharge success")
		time.Sleep(time.Second)
	}

	// 在alpine nocgo上使用ecdsa.PrivateKey会报错 msg="call ca login error" error="private key curve is not secp256k1"
	txHash, err := d.Login(key.Serialize(), userId, "")
	if err != nil {
		log.WithError(err).Error("call ca login error")
		c.JSON(200, withError(InternalError))
		return
	}

	log.WithFields(log.Fields{
		"userId": userId,
		"txHash": txHash,
	}).Info("user login")

	go func() {
		txHash, err = d.CheckApprove(key.Serialize())
		if err != nil {
			log.WithError(err).Error("call ca check approve error")
			return
		}
		if txHash.Hex() != (common.Hash{}).Hex() {
			log.WithFields(log.Fields{
				"userId": userId,
				"txHash": txHash,
			}).Info("user do approve tx")
		}
	}()

	c.JSON(200, withSuccess(""))
}
